# -*- coding: utf-8 -*-

import ctypes
try:
    from . import fastcomtec_types
except ImportError:
    import fastcomtec_types
import re
from typing import NamedTuple, List

class FastComTec():
    """ Hardware Class for the FastComtec MCS8. """

    def __init__(self, dll_path = r'C:\Windows\System32\DMCS8.dll', **kwargs) -> None:
        self.dll = ctypes.windll.LoadLibrary(dll_path)       
    ########################################
    # Fastcomtec Types
    ########################################
    def get_acq_status(self) -> fastcomtec_types.AcqStatus:
        status = fastcomtec_types.AcqStatus()
        self.dll.GetStatusData(ctypes.byref(status), 0)
        return status
    
    def get_acq_settings(self) -> fastcomtec_types.AcqSettings:
        settings = fastcomtec_types.AcqSettings()
        self.dll.GetSettingData(ctypes.byref(settings), 0)
        return settings
    
    def get_board_settings(self) -> fastcomtec_types.BoardSetting:
        boardsettings = fastcomtec_types.BoardSetting()
        self.dll.GetMCSSetting(ctypes.byref(boardsettings), 0)
        return boardsettings
    
    def get_acq_data(self) -> fastcomtec_types.AcqData:
        return NotImplementedError # code below doesn't give data for some reason
        data = fastcomtec_types.AcqData()
        self.dll.GetData(ctypes.byref(data), 0)
        return data
    ########################################
    # DLL Interaction
    ########################################
    def _send_dll_command(self, command: str) -> None:
        self.dll.RunCmd(0, bytes(command, ' utf-8'))

    def _send_dll_commands(self, commands: List[str]) -> None:
        for cmd in commands: 
            self._send_dll_command(cmd)
    ########################################
    # Measurement 
    ########################################
    def start_measurement(self) -> None:
        self._send_dll_command("start")

    def halt_measurement(self) -> None:
        self._send_dll_command("halt")
    
    def erase_measruement(self) -> None:
        self._send_dll_command("erase")
    ########################################
    # Saving and Loading Data and Config
    ########################################
    def load_config(self, file_path: str) -> None:
        """Doesn't have to be a .SET file, can also be a .mpa file."""
        self._send_dll_command(f"loadcnf {file_path}")

    def save_config(self) -> None:
        """Config stored in C:/mcs8x64/MCS8A.SET"""
        self._send_dll_command("savecnf") 

    def save_config_and_data_seperately(self, file_path: str, ) -> None:
        """
        NOTE: 
        + The Config gets saved twice, but both time with CHN1 as a header .
        + TOTALSUM, ROISUM, MAXVAL for Channel 2 might be totally wrong
        """
        self._send_dll_command(f"datname={file_path}_CHN1")
        self._send_dll_command("MC_A")
        self._send_dll_command("savedat")
        self._send_dll_command(f"datname={file_path}_CHN2")
        self._send_dll_command("MC_B")
        self._send_dll_command("savedat")

    def save_config_with_data(self, file_path: str) -> None:
        self._send_dll_command(f"mpaname={file_path}")
        self._send_dll_command("savempa")
    ########################################
    # Change Individual Settings
    ########################################
    def set_time_preset(self, time_preset_sec: float) -> None:
        self._send_dll_command(f"timepreset={time_preset_sec}")
    def set_sweep_preset(self, swpreset: int) -> None:
        self._send_dll_command(f"timepreset={swpreset}")
        

########################################
# Separate config and data
########################################    
def separate_config_and_data(file_path: str) -> None:
    path_split_up = file_path.split(".")
    path_split_up[-2] += "_config"
    config_path = ".".join(path_split_up) 

    path_split_up = file_path.split(".")
    path_split_up[-2] += "_data"
    data_path = ".".join(path_split_up) 
    
    regex = re.compile("(TDAT)")
    match = False
    with open(file_path, "r") as f:
        lines = f.readlines()
    write_data = False
    with open(config_path, "w+") as config_file:
        with open(data_path, "w+") as data_file:
            for line in lines:
                match = regex.search(line)
                if match:
                    write_data = True
                if write_data:
                    data_file.write(line)
                else:
                    config_file.write(line)

class BoundaryInfo(NamedTuple):
    boundary_id: str
    line_number: int

def separate_config_and_data_per_channel(file_path: str) -> None:
    path_split_up = file_path.split(".")
    path_split_up[-2] += "_config"
    config_path = ".".join(path_split_up) 
    
    # read in file
    with open(file_path, "r") as f:
        lines = f.readlines()
    
    boundaries: List[BoundaryInfo] = []

    # find boundaries of data sections
    regex_channel_data = re.compile("(TDAT)")
    match = False
    for i, line in enumerate(lines):
        match = regex_channel_data.search(line)
        if match:
            boundaries.append(BoundaryInfo(line.split(",")[0][1:], i)) # exclue "[" in data_id

    # write config file
    with open(config_path, "w+") as config_file:
        config_file.write("".join(lines[:boundaries[0].line_number]))

    # write data files
    for i in range(len(boundaries)):
        path_split_up = file_path.split(".")
        path_split_up[-2] += f"_data_{boundaries[i].boundary_id}"
        data_path = ".".join(path_split_up) 
        with open(data_path, "w+") as data_file:
            if (i+1) < len(boundaries):
                data_file.write("".join(lines[boundaries[i].line_number:boundaries[i+1].line_number]))
            else:
                data_file.write("".join(lines[boundaries[i].line_number:]))


